# CFLAGS and library paths for XAOL
# written 23 February 2000 by Blake Watters <blake@blake.dhs.org>
# Derived from XMMS m4 file - see credits below
# written 15 December 1999 by Ben Gertzfield <che@debian.org>

dnl Usage:
dnl AM_PATH_XAOL([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl
dnl Example:
dnl AM_PATH_XAOL(0.2.0, , AC_MSG_ERROR([*** XAOL >= 0.2.0 not installed - please install first ***]))
dnl
dnl Defines XAOL_CFLAGS, XAOL_LIBS, XAOL_DATA_DIR, XAOL_MODULE_DIR, 
dnl XAOL_CONNECTION_MODULE_DIR, XAOL_PROTOCOL_MODULE_DIR, 
dnl XAOL_PLUGIN_MODULE_DIR and XAOL_VERSION for your plugin pleasure.
dnl

dnl XAOL_TEST_VERSION(AVAILABLE-VERSION, NEEDED-VERSION [, ACTION-IF-OKAY [, ACTION-IF-NOT-OKAY]])
AC_DEFUN(XAOL_TEST_VERSION, [

# Determine which version number is greater. Prints 2 to stdout if	
# the second number is greater, 1 if the first number is greater,	
# 0 if the numbers are equal.						
									
# Written 15 December 1999 by Ben Gertzfield <che@debian.org>		
# Revised 15 December 1999 by Jim Monty <monty@primenet.com>		
									
    AC_PROG_AWK
    xaol_got_version=[` $AWK '						\
BEGIN {									\
    print vercmp(ARGV[1], ARGV[2]);					\
}									\
									\
function vercmp(ver1, ver2,    ver1arr, ver2arr,			\
                               ver1len, ver2len,			\
                               ver1int, ver2int, len, i, p) {		\
									\
    ver1len = split(ver1, ver1arr, /\./);				\
    ver2len = split(ver2, ver2arr, /\./);				\
									\
    len = ver1len > ver2len ? ver1len : ver2len;			\
									\
    for (i = 1; i <= len; i++) {					\
        p = 1000 ^ (len - i);						\
        ver1int += ver1arr[i] * p;					\
        ver2int += ver2arr[i] * p;					\
    }									\
									\
    if (ver1int < ver2int)						\
        return 2;							\
    else if (ver1int > ver2int)						\
        return 1;							\
    else								\
        return 0;							\
}' $1 $2`]								

    if test $xaol_got_version -eq 2; then 	# failure
	ifelse([$4], , :, $4)			
    else  					# success!
	ifelse([$3], , :, $3)
    fi
])

AC_DEFUN(AM_PATH_XAOL,
[
AC_ARG_WITH(xaol-prefix,[  --with-xmms-prefix=PFX  Prefix where XAOL is installed (optional)],
	xaol_config_prefix="$withval", xaol_config_prefix="")
AC_ARG_WITH(xaol-exec-prefix,[  --with-xaol-exec-prefix=PFX Exec prefix where XAOL is installed (optional)],
	xaol_config_exec_prefix="$withval", xaol_config_exec_prefix="")

if test x$xaol_config_prefix != x; then
    xaol_config_args="$xaol_config_args --prefix=$xaol_config_prefix"
    if test x${XAOL_CONFIG+set} != xset; then
  	XAOL_CONFIG=$xaol_config_prefix/bin/xaol-config
    fi
fi

AC_PATH_PROG(XAOL_CONFIG, xaol-config, no)
min_xaol_version=ifelse([$1], ,0.2.0, $1)

if test "$XAOL_CONFIG" = "no"; then
    no_xaol=yes
else
    XAOL_CFLAGS=`$XAOL_CONFIG $xaol_config_args --cflags`
    XAOL_LIBS=`$XAOL_CONFIG $xaol_config_args --libs`
    XAOL_VERSION=`$XAOL_CONFIG $xaol_config_args --version`
    XAOL_DATA_DIR=`$XAOL_CONFIG $xaol_config_args --data-dir`
    XAOL_MODULE_DIR=`$XAOL_CONFIG $xaol_config_args --module-dir`
    XAOL_CONNECTION_MODULE_DIR=`$XAOL_CONFIG $xaol_config_args \
					--connection-module-dir`
    XAOL_PLUGIN_MODULE_DIR=`$XAOL_CONFIG $xaol_config_args --plugin-module-dir`
    XAOL_PROTOCOL_PLUGIN_DIR=`$XAOL_CONFIG $xaol_config_args --protocol-module-dir`

    XAOL_TEST_VERSION($XAOL_VERSION, $min_xaol_version, ,no_xaol=version)
fi

AC_MSG_CHECKING(for XAOL - version >= $min_xaol_version)

if test "x$no_xaol" = x; then
    AC_MSG_RESULT(yes)
    ifelse([$2], , :, [$2])
else
    AC_MSG_RESULT(no)

    if test "$XAOL_CONFIG" = "no" ; then
	echo "*** The xaol-config script installed by XAOL could not be found."
      	echo "*** If XAOL was installed in PREFIX, make sure PREFIX/bin is in"
	echo "*** your path, or set the XAOL_CONFIG environment variable to the"
	echo "*** full path to xaol-config."
    else
	if test "$no_xaol" = "version"; then
	    echo "*** An old version of XAOL, $XAOL_VERSION, was found."
	    echo "*** You need a version of XAOL newer than $min_xaol_version."
	    echo "*** The latest version of XAOL is always available from"
	    echo "*** http://www.xaol.org/"
	    echo "***"

            echo "*** If you have already installed a sufficiently new version, this error"
            echo "*** probably means that the wrong copy of the xaol-config shell script is"
            echo "*** being found. The easiest way to fix this is to remove the old version"
            echo "*** of XAOL, but you can also set the XAOL_CONFIG environment to point to the"
            echo "*** correct copy of xaol-config. (In this case, you will have to"
            echo "*** modify your LD_LIBRARY_PATH enviroment variable, or edit /etc/ld.so.conf"
            echo "*** so that the correct libraries are found at run-time)"
	fi
    fi
    XAOL_CFLAGS=""
    XAOL_LIBS=""
    ifelse([$3], , :, [$3])
fi
AC_SUBST(XAOL_CFLAGS)
AC_SUBST(XAOL_LIBS)
AC_SUBST(XAOL_VERSION)
AC_SUBST(XAOL_DATA_DIR)
AC_SUBST(XAOL_MODULE_DIR)
AC_SUBST(XAOL_PROTOCOL_MODULE_DIR)
AC_SUBST(XAOL_PLUGIN_MODULE_DIR)
AC_SUBST(XAOL_CONNECTION_MODULE_DIR)
])
