/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// AOLSession.c

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "XMemory.h"
#include "AOLSession.h"
#include "ByteManip.h"
#include "AOLProtocol.h"
#include "Tokens.h"
#include "AOLStream.h"
#include "Events.h"
#include "AOLModules.h"

XAOLError ASNew (AOLSession** session) {
	AOLSession* s;
	XAOLError error;
	
	s = (AOLSession*) MAlloc(sizeof(AOLSession));
	if (s) {
		error = APGetInfo (&s->arch, &s->versionMajor, &s->versionMinor);
		if (error != No_Error) {
			MFree((void*) s);
			return (error);
			}
			
		s->checkValue 		= 130;
		s->status			= SS_Disconnected;
		s->P3Seq			= 0;
		s->P3Ack			= 0;
		s->heartBeat		= 0;
		s->screenname		= NULL;
		s->password			= NULL;
		s->connectionData 	= NULL;
		}
	
	*session = s;

	error = MStartModules (s);
	if (error != No_Error)
		return (error);
		
	return (No_Error);
	}
	
void ASDispose (AOLSession* session) {
	XAOLError error = No_Error;
	
	switch (session->status) {
		case SS_Disconnected:
			break;
		
		case SS_Connected:
		case SS_SigningOn:
			error = ASDisconnect (session);
			break;		
		
		case SS_Online:
			error = ASSignOff (session);
			break;
		}
	
	if (session->screenname != NULL)
		MFree(session->screenname);
	
	if (session->password != NULL)
		MFree(session->password);
	
	ASSessionKill (session);
	
	MFree(session);
	}

int ASIsValid (AOLSession* session) {
	return (session != NULL && session->checkValue == 130);
	}
		
XAOLError ASSendP3 (AOLSession* session, P3* p3) {
	char* buffer = NULL;
	int len;
	XAOLError error;
	
	if (session == NULL || session->status == SS_Disconnected || p3 == NULL)
    	return (PE_BadData);
		
	if (p3->type == P3_TYPE_INIT)
		session->P3Seq = session->P3Ack = 127;
	else if (p3->type == P3_TYPE_DATA) {
    	session->P3Seq = (session->P3Seq + 1) % 128;
    	if (session->P3Seq < 16)
			session->P3Seq = 16;
  		}

	p3->seq = session->P3Seq;
	p3->ack = session->P3Ack;
	
	error = P3BuildPacket(p3, &buffer, &len);
	XAOL_Assert_(error == No_Error);
	if (error == No_Error) {
		error = ASSend(session, buffer, len);
		XAOL_Assert_(error == No_Error);
		if (error != No_Error)
			return (error);
		}
	else
		return (error);
		
	return (No_Error);
	}
	
XAOLError ASReceiveP3 (AOLSession* session, P3* p3) {
	int p3datalen = p3->length - 3;
	XAOLError error;
	P3* tempP3;	
	char token[2];
	char data[1024];
	
	session->heartBeat++;
	session->P3Ack = p3->seq;
	
	fprintf(stderr, "Session status is %i, type is %i\n", session->status, p3->type);

	switch (session->status) {
		case SS_Connected:
			if (p3->type == P3_TYPE_ACK) {
				// Do the sign on
				char token[] = { 'D', 'd' };
				SignOnData sod;
				char t[2];
				char data[2000];
				
				error = APGetSignOnPacketData(&sod);
				if (error != No_Error)
					return (error);
					
				tempP3 = P3New(P3_TYPE_DATA);
				if (tempP3 == NULL)
					return (Mem_Error);
					
				error = P3Append(tempP3, token, sizeof(token));
				if (error != No_Error)
					return (error);
				
				error = P3Append(tempP3, sod.header, sod.hLength);
				if (error != No_Error)
					return (error);
					
				error = P3AppendChar(tempP3, (char) strlen(session->screenname));
				if (error != No_Error)
					return (error);
					
				error = P3Append(tempP3, session->screenname, strlen(session->screenname));
				if (error != No_Error)
					return (error);
					
				error = P3Append(tempP3, sod.snTrailer, sod.stLength);
				if (error != No_Error)
					return (error);

				error = P3AppendChar(tempP3, (char) strlen(session->password));
				if (error != No_Error)
					return (error);
					
				error = P3Append(tempP3, session->password, strlen(session->password));
				if (error != No_Error)
					return (error);
					
				error = P3Append(tempP3, sod.footer, sod.fLength);
				if (error != No_Error)
					return (error);

				error = ASSendP3(session, tempP3);
				if (error != No_Error)
					return (error);
				
				P3Dispose(tempP3);

				session->status = SS_SigningOn;
				}
			else {
				ASDisconnect(session);
				return (AE_InvalidData);
				}			
			break;
		
		case SS_SigningOn:
			switch (APCheckSignOn(p3)) {
				case AP_Success:
		  			session->status = SS_Online;
		  			break;
		  		
		  		case AP_BadPassword:
		  			ASDisconnect(session);
					return (AE_InvalidPassword);
		  			break;
		  		
		  		case AP_UnknownResponse:
		  			return (AE_SignOnErr);
		  			break;
		  		}
			break;
		
		case SS_Online:
/*			error = GetToken(p3, token, data);
			if (error != No_Error && error != TNoTokenData) {
				perror("Error getting token!\n");
				return (error);
				}

			if (error != TNoTokenData)
				printf("[xaol-2] Read Token %s\n", token);
*/			
			error = AStreamDispatch(session, p3);
			if (error != No_Error && error != TNoTokenData)
				return (error);
				
			switch (p3->type) {
				case P3_TYPE_HEARTBEAT:
					tempP3 = P3New(P3_TYPE_ACK);
					if (tempP3 == NULL)
						return (Mem_Error);
						
					error = ASSendP3(session, tempP3);
					
					P3Dispose(tempP3);
					session->heartBeat = 0;
					
					if (error != No_Error)
						return (error);							
					break;
				
				case P3_TYPE_DATA:
					if (session->heartBeat > 8) {
						tempP3 = P3New(P3_TYPE_ACK);
						if (tempP3 == NULL)
							return (Mem_Error);
						
						error = ASSendP3(session, tempP3);						
					
						P3Dispose(tempP3);
						session->heartBeat = 0;
						
						if (error != No_Error)
							return (error);
						}
					break;
				
				default:
					break;
				}
			break;
		
		default:
			perror("Unknown Session Status\n");
			break;
		}
	
	return (No_Error);
	}
	
XAOLError ASSignOn (AOLSession* session, char* screenname, char* password) {
	P3* p3 = NULL;
	XAOLError error;
	InitData initPacket;
	
	if (screenname == NULL || password == NULL)
		return (Arg_Error);
		
	session->screenname = (char*) MAlloc(strlen(screenname) + 1);
	strcpy(session->screenname, screenname);
	session->password = (char*) MAlloc(strlen(password) + 1);
	strcpy(session->password, password);
	
	error = APGetInitPacketData(&initPacket);
	if (error != No_Error)
		return (error);
		
	p3 = P3New(P3_TYPE_INIT);
	if (p3 == NULL) {
		XAOL_DEBUG_Print_("[XAOL] ASSignOn : P3 Error");
		return (Mem_Error);
		}
		
	error = P3AppendChar(p3, session->arch);
	if (error != No_Error) {
		P3Dispose(p3);
		return (error);
		}
		
	error = P3AppendChar(p3, session->versionMajor);
	if (error != No_Error) {
		P3Dispose(p3);
		return (error);
		}
		
	error = P3AppendChar(p3, session->versionMinor);
	if (error != No_Error) {
		P3Dispose(p3);
		return (error);
		}
		
	error = P3Append(p3, (char*) initPacket.data, initPacket.length);
	if (error != No_Error) {
		P3Dispose(p3);
		return (error);
		}
		
	error = ASSendP3(session, p3);
	if (error != No_Error) {
		P3Dispose(p3);
		return (error);
		}
		
	P3Dispose(p3);
	
	while (session->status == SS_Connected || session->status == SS_SigningOn)
		ASIdle();
	
	if (session->status == SS_Online) {
		printf("Dispatching Sign On Message.\n");
		XAEDispatch (kSignOnEvent, session, NULL);
		return (No_Error);
		}
	else
		return (AE_SignOnErr);
	}
	
XAOLError ASSignOff (AOLSession* session) {	
	if (! session)
    	return (AE_InvalidSession);
	
	XAEDispatch (kSignOffEvent, session, NULL);
	return (PutToken (session, "LO", NULL, 0));
	}
