/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// DataStream.h

#ifndef __DATA_STREAM__
#define __DATA_STREAM__

#include "XPlatform.h"
#include "XErrors.h"

// TODO - Portability definitions
#if TARGET_OS_UNIX == 1
	#include <netinet/in.h>
#else
	#include "in.h"
#endif

typedef struct DataStream {
	int checkValue;			// Private sanity check value
	char* buffer;			// The data in the stream
	char* p;				// Pointer to the location in the stream
	int position;			// The position we are in relative to the stream
	int end;				// The end of the stream (the length)
	int byteSwap;			// Remap the values to match network/host unsigned char ordering
	} DataStream;

DataStream*		 		DSNew 			(char* buffer, int length, int byteSwap);
int						DSIsValid		(DataStream* ds);
void					DSDispose		(DataStream* ds);
XAOLError 				DSGetBuffer 	(DataStream* ds, char** buffer);
XAOLError 				DSGetRaw		(DataStream* ds, unsigned char** buffer, int length);
XAOLError 				DSPutRaw		(DataStream* ds, unsigned char* data, int length);
XAOLError 				DSGetString 	(DataStream* ds, char** result);
XAOLError 				DSPutString 	(DataStream* ds, char* value);
XAOLError				DSGetByte		(DataStream* ds, unsigned char* result);
XAOLError 				DSPutByte 		(DataStream* ds, unsigned char value);
XAOLError				DSGetUInt8		(DataStream* ds, UInt8* result);
XAOLError 				DSPutUInt8	 	(DataStream* ds, UInt8 value);
XAOLError				DSGetUInt16		(DataStream* ds, UInt16* result);
XAOLError 				DSPutUInt16 	(DataStream* ds, UInt16 value);
XAOLError 				DSGetUInt32 	(DataStream* ds, UInt32* result);
XAOLError 				DSPutUInt32 	(DataStream* ds, UInt32 value);
XAOLError				DSGetSInt8		(DataStream* ds, SInt8* result);
XAOLError 				DSPutSInt8 		(DataStream* ds, SInt8 value);
XAOLError				DSGetSInt16		(DataStream* ds, SInt16* result);
XAOLError 				DSPutSInt16 	(DataStream* ds, SInt16 value);
XAOLError 				DSGetSInt32 	(DataStream* ds, SInt32* result);
XAOLError 				DSPutSInt32		(DataStream* ds, SInt32 value);
XAOLError 				DSPutDS 		(DataStream* ds, DataStream* append);
int 					DSGetLength 	(DataStream* ds);
int 					DSGetAvailable 	(DataStream* ds);
int 					DSGetUsed 		(DataStream* ds);
XAOLError 				DSAdvance 		(DataStream* ds, int byteCount);
XAOLError 				DSRewind 		(DataStream* ds, int byteCount);
XAOLError 				DSReset 		(DataStream* ds);

#endif __DATA_STREAM__