/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// Mod_AOLProtocol.c

#include "AOLProtocol.h"
#include "Modules.h"

XAOLError APLoadSymbols ();

// ** Function Pointer Types
typedef XAOLError (*APGetInfoPtr) (unsigned char* arch, unsigned char* majorVersion, unsigned char* minorVersion);
typedef XAOLError (*APGetInitPacketDataPtr) (InitData* packetData);
typedef XAOLError (*APGetSignOnPacketDataPtr) (SignOnData* sod); 
typedef int 	  (*APCheckSignOnPtr) (P3* p3);

// ** Globals
Module* protocolModule = NULL;

APGetInfoPtr apgetinfo = NULL;
APGetInitPacketDataPtr apgetinitpacketdata = NULL;
APGetSignOnPacketDataPtr apgetsignondata = NULL;
APCheckSignOnPtr apchecksignon = NULL;

XAOLError APGetInfo (unsigned char* arch, unsigned char* majorVersion, unsigned char* minorVersion) {
	if (! MIsValid(protocolModule))
		return (ME_InvalidModule);
	
	return (apgetinfo(arch, majorVersion, minorVersion));
	}

XAOLError APGetInitPacketData (InitData* packetData) {
	if (! MIsValid(protocolModule))
		return (ME_InvalidModule);
	
	return (apgetinitpacketdata(packetData));
	}
	
XAOLError APGetSignOnPacketData (SignOnData* sod) {
	if (! MIsValid(protocolModule))
		return (ME_InvalidModule);
	
	return (apgetsignondata(sod));
	}

int APCheckSignOn (P3* p3) {
	if (! MIsValid(protocolModule))
		return (ME_InvalidModule);
	
	return (apchecksignon(p3));
	}
	
XAOLError APSetProtocol (Module* module) {
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	if (module->type != Mod_Protocol)
		return (ME_WrongModuleType);
		
	protocolModule = module;

	return (APLoadSymbols());
	}
	
XAOLError APSetProtocolByName (char* protocol) {
	Module* module;
	XAOLError error;
	
	error = MGetModuleByName(protocol, &module);
	if (error != No_Error)
		return (error);
	
	if (module->type != Mod_Protocol)
		return (ME_WrongModuleType);
		
	protocolModule = module;
	
	return (APLoadSymbols());
	}

XAOLError APLoadSymbols () {
	XAOLError error;
	
	if (! MIsValid(protocolModule))
		return (ME_InvalidModule);
		
	// Load up all the symbols
	error = MGetSymbol (protocolModule, "Mod_APGetInfo", (void**) &apgetinfo);
	if (error != No_Error)
		return (error);
	
	error = MGetSymbol (protocolModule, "Mod_APGetInitPacketData", (void**) &apgetinitpacketdata);
	if (error != No_Error)
		return (error);
		
	error = MGetSymbol (protocolModule, "Mod_APGetSignOnPacketData", (void**) &apgetsignondata);
	if (error != No_Error)
		return (error);
	
	error = MGetSymbol (protocolModule, "Mod_APCheckSignOn", (void**) &apchecksignon);
	if (error != No_Error)
		return (error);
		
	return (No_Error);
	}