/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// Mod_AOLSession.c

#include "AOLSession.h"
#include "XErrors.h"
#include "Modules.h"

XAOLError 	ASLoadSymbols 				();

// ** Function Pointer Types
typedef XAOLError 	(*ASInitPtr)		();
typedef XAOLError 	(*ASShutdownPtr)	();
typedef XAOLError 	(*ASConnectPtr)		(AOLSession* session, void* connectionData);
typedef XAOLError 	(*ASDisconnectPtr)	(AOLSession* session);
typedef XAOLError 	(*ASSendPtr)		(AOLSession* session, char* buffer, int len);
typedef XAOLError 	(*ASReceivePtr)		(AOLSession* session, char* buffer, int len, int* read);
typedef void	  	(*ASSessionKillPtr)	(AOLSession* session);
typedef int		  	(*ASGetErrorPtr)	();
typedef void		(*ASIdlePtr)		();

// ** Globals
Module* connectionLayer = NULL;

ASInitPtr asinit = NULL;
ASShutdownPtr asshutdown = NULL;
ASConnectPtr asconnect = NULL;
ASDisconnectPtr asdisconnect = NULL;
ASSendPtr assend = NULL;
ASReceivePtr asreceive = NULL;
ASSessionKillPtr assessionkill = NULL;
ASGetErrorPtr asgeterror = NULL;
ASIdlePtr asidle = NULL;

// TODO - Error checking
// The Following Are Implemented within a Module
XAOLError ASInit () {	
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
	
	return (asinit());
	}
	
XAOLError ASShutdown () {
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);	
	
	return (asshutdown());
	}
	
XAOLError ASConnect (AOLSession* session, void* connectionData) {
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
		
	return (asconnect(session, connectionData));
	}
	
XAOLError ASDisconnect (AOLSession* session) {
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
	
	return (asdisconnect(session));
	}
	
XAOLError ASSend (AOLSession* session, char* buffer, int len) {	
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
	
	return (assend(session, buffer, len));
	}
	
XAOLError ASReceive (AOLSession* session, char* buffer, int len, int* read) {
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
	
	return (asreceive(session, buffer, len, read));
	}
	
void ASSessionKill (AOLSession* session) {
	if (! MIsValid(connectionLayer))
		return;
	
	assessionkill(session);
	}
	
int ASGetError () {
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
		
	return (asgeterror());
	}
	
void ASIdle () {
	if (! MIsValid(connectionLayer))
		return;
	
	asidle();
	}

XAOLError ASSetConnectionLayer (Module* module) {
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	if (module->type != Mod_ConnectionLayer)
		return (ME_WrongModuleType);
		
	connectionLayer = module;

	return (ASLoadSymbols());
	}
	
XAOLError ASSetConnectionLayerByName (char* layerName) {
	Module* module;
	XAOLError error;
	
	error = MGetModuleByName(layerName, &module);
	if (error != No_Error)
		return (error);
	
	if (module->type != Mod_ConnectionLayer)
		return (ME_WrongModuleType);
		
	connectionLayer = module;
	
	return (ASLoadSymbols());
	}

XAOLError ASLoadSymbols () {
	XAOLError error;
	
	if (! MIsValid(connectionLayer))
		return (ME_InvalidModule);
		
	// Load up all the symbols
	error = MGetSymbol(connectionLayer, "Mod_ASInit", (void**) &asinit);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASShutdown", (void**) &asshutdown);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASConnect", (void**) &asconnect);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASDisconnect", (void**) &asdisconnect);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASSend", (void**) &assend);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASReceive", (void**) &asreceive);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASSessionKill", (void**) &assessionkill);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASGetError", (void**) &asgeterror);
	if (error != No_Error)
		return (error);
	error = MGetSymbol(connectionLayer, "Mod_ASIdle", (void**) &asidle);
	if (error != No_Error)
		return (error);
	
	return (No_Error);
	}