/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// Modules.h

#ifndef __MODULES__
#define __MODULES__

#include "XErrors.h"

// Module Type
typedef enum ModType {
	Mod_Uninit,					// Module has not identified itself
	Mod_Protocol,				// Module provides a P3 protocol layer
	Mod_ConnectionLayer,		// Module provides a method for connecting to AOL host
	Mod_Plugin					// Module implements some functionality, extending the library
	} ModType;

// Define the maximum number of messages supported
#define kMaxMessages 999

enum {
	kModuleNameLen  = 64,
	kVersionLen		= 32,
        kDescriptionLen = 128
};

typedef struct Module {
	// Module Information (Application Developer Side)
	char name[kModuleNameLen];			// The name of this module	
	long version;						// The version of this module
	ModType type;						// The type of this module
	int messages[kMaxMessages];			// Messages this module wants to get
        char description[kDescriptionLen];              // Description of the module
   
	// Implementation (private) data
	void* data;							// Pointer to implementation dependent data
	int id;								// The ID number of this module (internally assigned)
	int checkValue;						// Private sanity check value
	
	// Module List
	struct Module* next;				// Pointer to the next module in the current context
	struct Module* previous;			// Pointer to the previous module in the current context
	} Module;

// Basic Module Symbol Names and Function Pointer types
#define kModInit 	"Module_Init"
#define kModExit 	"Module_Exit"

typedef XAOLError (*ModInitExitPtr)	(Module*);

// Portable
Module*		MNew					();
void		MDispose				(Module* module);
int 		MIsValid 				(Module* module);
XAOLError 	MAdd		 			(Module* module);
XAOLError 	MRemove		 			(Module* module);
XAOLError	MDispatchMessage		(int messageType, void* messageData);
int			MNumModules				();
XAOLError	MGetModuleByID			(int id, Module** module);
XAOLError	MGetModuleByName		(char* name, Module** module);
XAOLError	MGetModuleByKind		(ModType type, Module** module);
XAOLError	MLoad					(Module** module, char* path);
XAOLError	MUnload					(Module* module);

// Platform Specific
XAOLError	MPrivateLoad			(Module* module, char* path);
XAOLError	MPrivateUnload			(Module* module);
XAOLError 	MLoadDirectory	 		(char* directory);
XAOLError	MSendMessage			(Module* module, int messageType, void* messageData);
int			MGetError				();
XAOLError	MGetSymbol				(Module* module, char* symbol, void** symAddr);

// Module Entry and Exit
XAOLError	Module_Init				(Module* module);
XAOLError	Module_Exit				(Module* module);

#endif __MODULES__