/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// PacketParser.c

#include <stdlib.h>
#include <sys/types.h>

#include "XMemory.h"
#include "DataStream.h"
#include "PacketParser.h"

PacketData PDPCreate (void) {
	PacketData pdp;

	pdp = MAlloc(sizeof(PacketData));
	pdp->num = 0;
	pdp->pe = NULL;

	return (pdp);
	}
	
void PDPDestroy (PacketData* pdp) {
	PacketElement pe, next = NULL;

	if ((* pdp) != NULL) {
		for (pe = (*pdp)->pe; pe; pe = next) {
			next = pe->next;

			MFree(pe->data);
			MFree(pe);
			}

		MFree(*pdp);
		(*pdp) = NULL;
  		}
	}

short PDPGetLength (PacketData pdp, short type) {
	PacketElement pe;

	if (pdp != NULL) {
		for (pe = pdp->pe; pe; pe = pe->next)
		if (pe->type == type)
			return (pe->length);
		}
		
	return (-1);
	}
	
PacketElement PDPIsPresent (PacketData pdp, short type) {
	PacketElement pe;

	if (pdp != NULL) {
		for (pe = pdp->pe; pe; pe = pe->next) {
			if (pe->type == type)
				return (pe);
			}
		}
		
	return (NULL);
	}
	
int PDPAdd (PacketData pdp, short type, short length, void* data) {
	PacketElement pe;

	pe = MAlloc(sizeof(PacketElement));
	if (pe == NULL)
		return (-1);

	printf("Got the PacketElement\n");

	pe->data   = MAlloc(length + 25);
	if (pe->data == NULL)
		return (-1);

	printf("Allocated the data buffer\n");

	pe->type   = type;
	pe->length = length;
	pe->next   = pdp->pe;

	printf("Copied the other values\n");
	memcpy(pe->data, (unsigned char*) data, length + 1);

	pdp->pe = pe;
	pdp->num++;

	return (1);
	}
	
void PDPWrite (PacketData pdp, DataStream* ds) {
	PacketElement pe;

	if (pdp != NULL && ds != NULL) {
		for (pe = pdp->pe; pe; pe = pe->next) {
			DSPutUInt16(ds, pe->type);
			DSPutUInt16(ds, pe->length);
			DSPutRaw(ds, pe->data, pe->length);
    		}
    	}
	}
	
PacketData PDPRead (DataStream* ds) {
	PacketData pdp = NULL;
	short type, length;
	unsigned char* value;
	int i;
	XAOLError error;

	if (ds != NULL) {
		pdp = PDPCreate();

		printf("Got new PDP\n");

    		while((i = DSGetAvailable(ds)) > 0) {
			printf("Reading the type\n");
			error   = DSGetByte(ds, (short*) &type);
			printf("Reading the length\n");
			error = DSGetByte(ds, (short*) &length);
			printf("Reading the value..\n");
			error  = DSGetRaw(ds, &value, length);
			printf("Adding the whole thing\n");
			PDPAdd(pdp, type, length, value);
			printf("Freeing the memory used.\n");
			MFree(value);
			}

		return (pdp);
  		}
  		
	return (NULL);
	}

short PDPGet16 (PacketData pdp, short type) {
	PacketElement pe = PDPIsPresent(pdp, type);
	short rv = 0;

	if (pe != NULL) {
		memcpy((unsigned char *)& rv, pe->data, sizeof(short));
		rv = ntohs(rv);
		}

	return (rv);
	}
	
int	PDPPut16 (PacketData pdp, short type, short val) {
	short ival = htons(val);

	return (PDPAdd(pdp, type, sizeof(short), (short *)& ival));
	}

unsigned short PDPGetU16 (PacketData pdp, short type) {
	PacketElement pe = PDPIsPresent(pdp, type);
	unsigned short rv = 0;

	if (pe != NULL) {
		memcpy((unsigned char*)& rv, pe->data, sizeof(unsigned short));
		rv = ntohs(rv);
  		}

	return (rv);
	}
	
int PDPPutU16 (PacketData pdp, short type, unsigned short val) {
	unsigned short ival = htons(val);

	return (PDPAdd(pdp, type, sizeof(unsigned short), (unsigned short*)& ival));
	}

long PDPGet32 (PacketData pdp, short type) {
	PacketElement pe = PDPIsPresent(pdp, type);
	long rv = 0;

	if (pe != NULL) {
		memcpy((unsigned char*)& rv, pe->data, sizeof(unsigned short));
		rv = ntohl(rv);
		}

	return (rv);
	}
	
int PDPPut32 (PacketData pdp, short type, long val) {
	long ival = htonl(val);

	return (PDPAdd(pdp, type, sizeof(long), (long*)& ival));
	}

unsigned long PDPGetU32 (PacketData pdp, short type) {
	PacketElement pe = PDPIsPresent(pdp, type);
	unsigned long rv = 0;

	if (pe != NULL) {
		memcpy((unsigned char*)& rv, pe->data, sizeof(unsigned long));
		rv = ntohl(rv);
		}

	return (rv);
	}
	
int PDPPutU32 (PacketData pdp, short type, unsigned long val) {
	unsigned long ival = htonl(val);

	return (PDPAdd(pdp, type, sizeof(unsigned long), (long *)& ival));
	}

unsigned char*	PDPGetRaw (PacketData pdp, short type) {
	PacketElement pe = PDPIsPresent(pdp, type);
	unsigned char* rv = NULL;

	if (pe != NULL) {
    	rv = (unsigned char*) MAlloc(pe->length);
    	memcpy(rv, pe->data, pe->length);
		}

	return (rv);
	}
	
int PDPPutRaw (PacketData pdp, short type, short length, unsigned char* data) {
	if (data != NULL)
		return (PDPAdd(pdp, type, length, data));
	else
		return (-1);
	}

char* PDPGetString (PacketData pdp, short type) {
	PacketElement pe = PDPIsPresent(pdp, type);
	char* rv = NULL;

	if (pe != NULL) {
		rv = (char *) MAlloc(pe->length + 1);
		memcpy(rv, pe->data, pe->length);
		rv[pe->length] = '\0';
		}

	return (rv);
	}
	
int PDPPutString (PacketData pdp, short type, char* data) {
	if (data != NULL)
		return (PDPAdd(pdp, type, strlen(data), (char *) data));
    else
		return(-1);
	}
