/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// UnixModules.c

#include <stdlib.h>
#include <stdio.h>
#include <dlfcn.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>

#include "XPlatform.h"
#include "XErrors.h"
#include "Modules.h"

int error = 0;

typedef struct UnixPrivateMData {
	void* handle;
	void* main;
} UnixPrivateMData;

XAOLError MPrivateLoad (Module* module, char* path) {
	UnixPrivateMData* umd;
	
	if (! strlen(path))
		return (Arg_Error);
		
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	umd = (UnixPrivateMData*) malloc (sizeof(UnixPrivateMData));
	if (umd == NULL) {
		XAOL_DEBUG_Print_("Error making module ref\n");
		return (Mem_Error);
		}
	
	umd->handle = NULL;
	umd->main = NULL;
	
	umd->handle = dlopen(path, RTLD_LAZY);
	if (umd->handle == NULL) {
		fputs(dlerror(), stderr);
		perror("\n");
		return (ME_InternalError);
		}
		
	module->data = (void*) umd;

	return (No_Error);
	}

XAOLError MPrivateUnload (Module* module) {
	UnixPrivateMData* umd;
	char* err;
	
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	umd = (UnixPrivateMData*) module->data;
	
	dlclose(umd->handle);
	if ((err = dlerror()) != NULL);
		return (ME_InternalError);

	free(umd);
	
	return (No_Error);
	}

XAOLError MLoadDirectory (char* directory) {
	DIR* dp;
	char* currentDir = NULL;
	struct dirent* entry;
	struct stat statbuf;
	char extension[3];
	Module* module = NULL;
	XAOLError error;
	
	if (directory == NULL)
		return (Arg_Error);
		
	currentDir = getcwd(currentDir, 0);
	
	if ((dp = opendir(directory)) == NULL) {
		if (currentDir != NULL)
			free(currentDir);		
		return (File_Error);
		}
	
	chdir(directory);
	
	while ((entry = readdir(dp)) != NULL) {
		stat(entry->d_name, &statbuf);
		
		if (S_ISDIR(statbuf.st_mode)) {
			if (strcmp(".", entry->d_name) == 0 ||
				strcmp("..", entry->d_name) == 0)
				continue;
			return (MLoadDirectory(entry->d_name));
			}
		
		printf("File found: %s (extension is %s)", entry->d_name, entry->d_name + (strlen(entry->d_name) - 3));
		snprintf(extension, 3, "%s", entry->d_name + (strlen(entry->d_name) - 3));
		if (strcmp(".so", extension)) {
			module = MNew();
			error = MLoad(&module, entry->d_name);
			}
		}
	
	closedir(dp);
	
	if (currentDir != NULL) {
		chdir(currentDir);
		free(currentDir);
		}
	
	return (No_Error);
	}

XAOLError MSendMessage (Module* module, int messageType, void* messageData) {
	UnixPrivateMData* umd;
	XAOLError err;
	
	if (! MIsValid(module))
		return (ME_InvalidModule);
	
	umd = (UnixPrivateMData*) module->data;
	if (umd == NULL)
		return (Mem_Error);
	
	// If main hasn't been looked up yet, look it up
	if (umd->main == NULL) {
		err = MGetSymbol(module, "main", &umd->main);
		if (err != No_Error) {
			XAOL_DEBUG_Print_("Error looking up the main symbol.\n");
			return (err);
			}
		}

	((void (*) (int, void*)) umd->main) (messageType, messageData);
	return (No_Error);
	}

int MGetError () {
	int lerror;
	
	lerror = error;
	error = 0;
	
	return (lerror);
	}

XAOLError MGetSymbol (Module* module, char* symbol, void** symAddr) {
	char* err;
	UnixPrivateMData* umd;

	if (! MIsValid(module))
		return (ME_InvalidModule);

	umd = (UnixPrivateMData*) module->data;
	
	*symAddr = dlsym(umd->handle, symbol);
	if ((err = dlerror()) != NULL) {
		perror(err);
		return (ME_InternalError);
		}
		
	return (No_Error);
	}
