/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// XPlatform.h

#ifndef __XPLATFORM__
#define __XPLATFORM__

/*
		TARGET_CPU_PPC			- Compiler is generating PowerPC instructions
		TARGET_CPU_X86			- Compiler is generating x86 instructions

		TARGET_OS_MAC			- Generate code will run under Mac OS
		TARGET_OS_WIN32			- Generate code will run under 32-bit Windows
		TARGET_OS_UNIX			- Generate code will run under some unix 
		TARGET_OS_BEOS			- Generate code will run under BeOS
		
		TARGET_RT_LITTLE_ENDIAN	- Generated code uses little endian format for integers
		TARGET_RT_BIG_ENDIAN	- Generated code uses big endian format for integers 	
*/

/***********************************************************************
***********************************************************************/
/* Compiler & OS */
#if defined(__MWERKS__)
	/* This is CodeWarrior */
	#define COMPILER_MWCW		1
	#define COMPILER_MWCWPRO4 	0 // TODO!!! -- this should NOT be hardcoded
	#define COMPILER_MWCWPRO5 	1 // TODO!!! -- this should NOT be hardcoded
	#define COMPILER_EGCS   	0
	#define COMPILER_MSVC   	0
	
	#include <mslGlobals.h>

	#if __dest_os == __mac_os
		/* This is MacOS and CodeWarrior */
		#define TARGET_OS_MAC	1
		#define TARGET_OS_WIN32	0
		#define TARGET_OS_UNIX	0
		#define TARGET_OS_BEOS	0
		#if !defined(__CONDITIONALMACROS__)
			#include <ConditionalMacros.h>
		#endif /* !defined(__CONDITIONALMACROS__) */
	#elif __dest_os == __win32_os
		/* This is Win32 and CodeWarrior */
		#define TARGET_OS_MAC	0
		#define TARGET_OS_WIN32	1
		#define TARGET_OS_UNIX	0
		#define TARGET_OS_BEOS	0
	#else
		#error MetroWerks CodeWarrior detected on unsupport platform.
	#endif
	/* This needs to have -D__linux__ switches passed to compiler */
#elif defined(__linux__) || defined(__LINUX__) || (defined(__dest_os_) && __dest_so == __linux_os) || defined(LINUX)
	/* This is Linux and egcs (we're assuming) */
	#define COMPILER_MWCW 		0
	#define COMPILER_MWCWPRO4 	0
	#define COMPILER_MWCWPRO5 	0
	#define COMPILER_EGCS   	1
	#define COMPILER_MSVC   	0

	#define TARGET_OS_MAC	0
	#define TARGET_OS_WIN32	0
	#define TARGET_OS_UNIX	1
	#define TARGET_OS_BEOS	0
#elif defined(_MSC_VER) && defined(_WIN32)
	/* This is Visual C++ on Win32 */
	#define COMPILER_MWCW 		0
	#define COMPILER_MWCWPRO4 	0
	#define COMPILER_MWCWPRO5 	0
	#define COMPILER_EGCS   	0
	#define COMPILER_MSVC   	1

	#define TARGET_OS_MAC	0
	#define TARGET_OS_WIN32	1
	#define TARGET_OS_UNIX	0
	#define TARGET_OS_BEOS	0
#else
	/* TODO -- BeOS */
	#error ERROR -- unknown OS
#endif /* defined(__MWERKS__) */

/***********************************************************************
***********************************************************************/
/* CPU & Endianess */
#if TARGET_OS_MAC == 1
	/* Nothing to do for this case -- ConditionalMacros.h took care of it for us. */
#elif TARGET_OS_UNIX == 1 /* assuming a GCC compliant compiler */
	#if #cpu(powerpc)
		#define TARGET_CPU_PPC 1
		#define TARGET_CPU_X86 0
		
		#define TARGET_RT_LITTLE_ENDIAN 0
		#define TARGET_RT_BIG_ENDIAN 1
		
	#elif #cpu(i386)
		#define TARGET_CPU_PPC 0
		#define TARGET_CPU_X86 1

		#define TARGET_RT_LITTLE_ENDIAN 1
		#define TARGET_RT_BIG_ENDIAN 0
	#else
		#error ERROR -- unknown CPU
	#endif
#elif TARGET_OS_WIN32 == 1
	/* Of course we could be running Win32 on PowerPC or Alpha but I doubt it somehow... */
	#define TARGET_CPU_PPC 0
	#define TARGET_CPU_X86 1

	#define TARGET_RT_LITTLE_ENDIAN 1
	#define TARGET_RT_BIG_ENDIAN 0
#else
	/* TODO -- Win32 & BeOS */
	#error ERROR -- unknown OS
#endif

/***********************************************************************
***********************************************************************/
/* Now include types */
#if TARGET_OS_MAC == 1
	#include "MacOSTypes.h"
#elif TARGET_OS_UNIX == 1
	#include "UnixTypes.h"
#elif TARGET_OS_WIN32 == 1
	#include "Win32Types.h"
#else
	#error ERROR -- unknown OS
#endif

#endif __XPLATFORM__