/***************************************************************************
* Copyright (C)1999 The XAOL Project, All Rights Reserved.                 *
* WWW: http://www.xaol.org/                                                *
***************************************************************************/
// IPTunnel.c

#include "Plugin.h"
#include "Events.h"
#include "Tokens.h"
#include "AOLStream.h"
#include "DataStream.h"
#include "PacketParser.h"

#include <string.h>
#include <stdio.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

Module* thisModule;

#define kVersion 0.2

void IPParser (XAOLEvent event);
void RequestIP (AOLSession* session);
void SetupIP (AOLSession*, P3* p3, void*);
void DataIn (AOLSession*, P3* p3, void*);

XAOLError Module_Init (Module* module) {
	printf("Tunneling Module Initialized!\n");
	
	thisModule = module;
	snprintf(thisModule->name, kModuleNameLen, "IP Tunnel");
        snprintf(thisModule->description, kDescriptionLen, "Access the internet through America Online");
	thisModule->version = kVersion;
	thisModule->type = Mod_Plugin;
	
	return (No_Error);
	}

XAOLError Module_Start (Module* module, AOLSession* session) {
	XAOLError error;

	printf("[IT] Module Started!\n");
	error = XAERegister (IPParser);
	
	return (error);
	}
	
void IPParser (XAOLEvent event) {
	printf("[IT] Parser Called!\n");
	
	switch (event.what) {
		case kSignOnEvent:
			printf("Got Sign on Event - sending tokens.\n");
			RequestIP(event.where);
			break;
		
		case kSignOffEvent:
			break;
		
		default:
			break;
		}
	}
	
void RequestIP (AOLSession* session) {
	XAOLError error;
	char ipInitToken[] = {121, 97};
	
	error = PutToken (session, ipInitToken, NULL, 0);
	if (error == No_Error) {
		printf("Tokens sent - awaiting responses.\n");		
		error = AStreamAdd(session, "ya", SetupIP, NULL);
		if (error != No_Error) {
			printf("[IP Tunnel] Error adding stream : %i\n", error);
			return;
			}

		error = AStreamAdd(session, "yc", DataIn, NULL);
		if (error != No_Error)
			printf("[IP Tunnel] Error adding stream : %i\n", error);
		}
	else {
		printf("Error sending tokens (%i)\n", error);
		// TODO - Error handling
		}
	}

void SetupIP (AOLSession* session, P3* p3, void* data) {
	DataStream* ds;
	char* hostname;
	XAOLError error;
	struct in_addr aolip, dnsip;
	PacketData pdp;

	ds = DSNew(p3->data + 2, p3->length - 2, true);
	if (ds == NULL)
	//	 TODO - Error handling
		return;
	
	printf("[xaol] Got AOL IP Response!\n");
	
	pdp = PDPRead(ds);

	printf("Back from PDPRead()\n");
	hostname = PDPGetString(pdp, 10);
	aolip.s_addr = ntohl(PDPGetUInt32(pdp, 3));
	dnsip.s_addr = ntohl(PDPGetUInt32(pdp, 4));

	printf("IP Tunneling: AOL Hostname = %s\n", hostname);
	printf("IP Tunneling: AOL IP Address = %s\n", inet_ntoa(aolip));
	printf("IP Tunneling: AOL DNS Server = %s\n", inet_ntoa(dnsip));
	fflush(stdout);

	PDPDestroy(&pdp);
	DSDispose(ds);
	}

void DataIn (AOLSession* session, P3* p3, void* data) {
	printf("Data in!\n");
	}
